/*
 * fritzbox.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include "fritzlistener.h"
#include "fritzfonbuch.h"
#include "i18n.h"
#include "setup.h"
#include "menu.h"

static const char *VERSION        = "0.0.5";
static const char *DESCRIPTION    = "Fritz!Box Plugin for AVM Fritz!Box";
static const char *MAINMENUENTRY  = "Fritz!Box phone book";

class cPluginFritzbox : public cPlugin {
private:
	cFritzFonbuch *fonbuch;
	cFritzListener *listener;
public:
	cPluginFritzbox(void);
	virtual ~cPluginFritzbox();
	virtual const char *Version(void) { return VERSION; }
	virtual const char *Description(void) { return DESCRIPTION; }
	virtual const char *CommandLineHelp(void);
	virtual bool ProcessArgs(int argc, char *argv[]);
	virtual bool Initialize(void);
	virtual bool Start(void);
	virtual void Stop(void);
	virtual void Housekeeping(void);
	virtual void MainThreadHook(void);
	virtual cString Active(void);
	virtual time_t WakeupTime(void);
	virtual const char *MainMenuEntry(void) { return fritzboxConfig.hideMainMenu ? NULL : tr(MAINMENUENTRY); }
	virtual cOsdObject *MainMenuAction(void);
	virtual cMenuSetupPage *SetupMenu(void);
	virtual bool SetupParse(const char *Name, const char *Value);
	virtual bool Service(const char *Id, void *Data = NULL);
	virtual const char **SVDRPHelpPages(void);
	virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
};

cPluginFritzbox::cPluginFritzbox(void)
{
	// Initialize any member variables here.
	// DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
	// VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginFritzbox::~cPluginFritzbox()
{
	// Clean up after yourself!
}

const char *cPluginFritzbox::CommandLineHelp(void)
{
	// Return a string that describes all known command line options.
	return NULL;
}

bool cPluginFritzbox::ProcessArgs(int argc, char *argv[])
{
	// Implement command line argument processing here if applicable.
	return true;
}

bool cPluginFritzbox::Initialize(void)
{
	// Initialize any background activities the plugin shall perform.
	return true;
}

bool cPluginFritzbox::Start(void)
{
	// Start any background activities the plugin shall perform.
	RegisterI18n(Phrases);
	fonbuch  = new cFritzFonbuch();
	listener = new cFritzListener(fritzboxConfig.url, PORT_MONITOR, fonbuch);
	listener->Start();
	return true;
}


void cPluginFritzbox::Stop(void)
{
	// Stop any background activities the plugin shall perform.
	delete listener;
	delete fonbuch;
}

void cPluginFritzbox::Housekeeping(void)
{
	// Perform any cleanup or other regular tasks.
}

void cPluginFritzbox::MainThreadHook(void)
{
	char *msg = listener->GetMessage();
	if (msg) {
		Skins.Message(mtInfo, msg);
	}
}

cString cPluginFritzbox::Active(void)
{
	// Return a message string if shutdown should be postponed
	return NULL;
}

time_t cPluginFritzbox::WakeupTime(void)
{
	// Return custom wakeup time for shutdown script
	return 0;
}

cOsdObject *cPluginFritzbox::MainMenuAction(void)
{
	if (fonbuch->Active()) {
		Skins.Message(mtError, tr("Phone book not available yet"));
		return NULL;
	}
	else
		return new cMenuFritzbox( fonbuch );
}

cMenuSetupPage *cPluginFritzbox::SetupMenu(void)
{
	// Return a setup menu in case the plugin supports one.
	return new cMenuSetupFritzbox( );
}

bool cPluginFritzbox::SetupParse(const char *Name, const char *Value)
{
	// Parse your own setup parameters and store their values.
	return fritzboxConfig.SetupParse(Name, Value);
}

bool cPluginFritzbox::Service(const char *Id, void *Data)
{
	// Handle custom service requests from other plugins
	return false;
}

const char **cPluginFritzbox::SVDRPHelpPages(void)
{
	// Return help text for SVDRP commands this plugin implements
	return NULL;
}

cString cPluginFritzbox::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
	// Process SVDRP commands this plugin implements
	return NULL;
}

VDRPLUGINCREATOR(cPluginFritzbox); // Don't touch this!
