#include "menu.h"
#include "setup.h"

cMenuFritzbox::cMenuFritzbox( cFritzFonbuch *fonbuch )
:cOsdMenu("Fritz!Box", 1) // just dummy values
{
	this->fonbuch = fonbuch;
	reloadCallList = true;
	DisplayFonbuch();
}               

cMenuFritzbox::~cMenuFritzbox()
{
}

eOSState cMenuFritzbox::ProcessKey (eKeys Key) {
	std::string numberToCall;
	eOSState state = osUnknown;
	if (state == osUnknown) {
		switch (Key) {
		case kOk:
			switch (currentMode) {
			case FONBUCH:
				numberToCall = fonbuch->RetrieveFonbuchEntry(Current())->number;
				break;
			case IN:
				numberToCall = fonbuch->RetrieveCallListEntry(cFritzFonbuch::INCOMING, Current())->remoteNumber;
				break;
			case OUT:
				numberToCall = fonbuch->RetrieveCallListEntry(cFritzFonbuch::OUTGOING, Current())->remoteNumber;
				break;
			case MISSED:
				numberToCall = fonbuch->RetrieveCallListEntry(cFritzFonbuch::MISSED, Current())->remoteNumber;
				break;
			}	
			if (numberToCall.empty())
				Skins.Message(mtError, tr("No number to call"));
			else
				if (fonbuch->InitCall(numberToCall))
					Skins.Message(mtInfo, tr("Pick up your phone now"));
				else
					Skins.Message(mtError, tr("Error while initiating call"));
			state = osContinue;
			break;
		case kRed:
			if (currentMode == FONBUCH) {
				fonbuch->Start();
				Skins.QueueMessage(mtInfo, tr("Retrieving phone book"));
				state = osEnd;				
			} else {
				DisplayFonbuch();
				state = osContinue;
			}
			break;
		case kGreen:
		case kYellow:
		case kBlue:
			if (reloadCallList)
				Skins.Message(mtStatus, tr("Please wait..."));
			if (!reloadCallList || fonbuch->LoadCallList()) {
				reloadCallList = false;
				DisplayCalls((cFritzFonbuch::callType)(Key - kRed));
			} else {
				Skins.Message(mtError, tr("Error fetching the call list"));
			}
			state = osContinue;
			break;
		default:
			break;
		}
	}
	if (state == osUnknown)
		state = cOsdMenu::ProcessKey(Key);
	return state;
}

void cMenuFritzbox::DisplayFonbuch() {
	currentMode = FONBUCH;
	SetTitle(tr("Fritz!Box phone book"));
	SetCols(20);
	Clear();
	for (unsigned int pos=0; pos < fonbuch->GetFonbuchSize(); pos++) {
		cFonbuchEntry *fe = fonbuch->RetrieveFonbuchEntry(pos);
		// build the menu entries
		char *line;
		asprintf(&line,"%s\t%s",fe->number.c_str(), fe->name.c_str());
		Add(new cOsdItem(line));
	}
	SetHelp(tr("Reload"), "|<-", "?|<-", "|->");
	Display();
}

void cMenuFritzbox::DisplayCalls(cFritzFonbuch::callType ct) {
	currentMode = (mode) ct;
	std::string title=tr("Fritz!Box call list");
	Clear();
	title += " (";
	switch(ct) {
	case cFritzFonbuch::INCOMING:
		SetCols(14, 5);
		title += tr("incoming");
	break;
	case cFritzFonbuch::MISSED:
		SetCols(14);
		title += tr("missed");
	break;
	case cFritzFonbuch::OUTGOING:
		SetCols(14, 5);
		title += tr("outgoing");
	break;
	}
	title += ")";
	for (unsigned int pos=0; pos < fonbuch->GetCallListSize(ct); pos++) {
		cCallEntry *ce = fonbuch->RetrieveCallListEntry(ct, pos);
		// build the menu entries
		char *line;
		char *buffer;

		// show local number if enabled		
		if (fritzboxConfig.showNumberInCallList == true) {
			char sep;
			if (ct == cFritzFonbuch::MISSED || ct == cFritzFonbuch::INCOMING)
				sep = '>';
			else
				sep = '<';
			// show remote name, remote number or "unknown" 
			asprintf(&buffer, "%s %c %s", 
					ce->remoteName.length()   > 0 ? ce->remoteName.c_str()  : 
					ce->remoteNumber.length() > 0 ? ce->remoteNumber.c_str(): 
					tr("unknown"),						
					sep,
					ce->localNumber.c_str());
		}
		else {
			// show remote name, remote number or "unknown" 
			asprintf(&buffer, "%s", 
					ce->remoteName.length()   > 0 ? ce->remoteName.c_str()  : 
					ce->remoteNumber.length() > 0 ? ce->remoteNumber.c_str(): 
					tr("unknown"));						
		}
		if (ct == cFritzFonbuch::MISSED)
			asprintf(&line,"%s\t%s",
					ce->date.c_str(),
					buffer);
		else
			asprintf(&line,"%s\t%s\t%s",
					ce->date.c_str(),
					ce->duration.c_str(),
					buffer);
		Add(new cOsdItem(line));	
		free(buffer);
		free(line);
	}	
	SetTitle(title.c_str());
	Display();
	SetHelp(tr("PB"), "|<-", "?|<-", "|->");
}
