#include "tcpclient.h"
#include <vdr/tools.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>

#include <string>
#include <sstream>

cTcpClient::cTcpClient(std::string &hostname, int port) {
	connected = false;
	this->hostname = hostname;
	this->port = port;
}

cTcpClient::~cTcpClient() {
	close(fd);
}

void cTcpClient::Connect() {
	if (connected) {
		close(fd);
		connected = false;
	}

	struct hostent *host;
	struct sockaddr_in addr;

	if (!inet_aton(hostname.c_str(), &addr.sin_addr)) {
		host = gethostbyname(hostname.c_str());
		if (!host)
			throw cTcpException(cTcpException::ERR_HOST_NOT_RESOLVABLE);
		addr.sin_addr = *(struct in_addr*)host->h_addr;
		dsyslog("tcpclient.c: Resolved hostname.");
	}

	fd = socket(PF_INET, SOCK_STREAM, 0);

	if (fd < 0)
		throw cTcpException(cTcpException::ERR_SOCKET_CREATE);
	dsyslog("tcpclient.c: Created socket.");
	addr.sin_port = htons(port);
	addr.sin_family = AF_INET;
	if (connect(fd, (struct sockaddr*)&addr, sizeof(addr))< 0)
		throw cTcpException(cTcpException::ERR_HOST_NOT_REACHABLE);
	dsyslog("tcpclient.c: Successfully connected.");
	connected = true;
}

// ----- Stuff for reading from socket ----------------------------------------

std::string cTcpClient::Read() {
	if (!connected)
		Connect();
	int size = recv(fd, buffer, TCP_BUF_SIZE - 1, 0);
	if (size == 0) {
		connected = false;
		throw cTcpException(cTcpException::ERR_CONNECTION_RESET);
	}
	buffer[size] = '\0';
	return std::string(buffer);
}

cTcpClient &cTcpClient::operator>>(std::ostringstream &ss) {
	ss << Read();
	return *this;
}

cTcpClient &cTcpClient::operator>>(std::string &s) {
	s += Read();
	return *this;
}

// ----- Stuff for writing to socket ------------------------------------------

void cTcpClient::Write(std::string s) {
	if (!connected)
		Connect();
	int size = send(fd, s.c_str(), s.length(), 0);
	dsyslog("tcpclient.c: sending %s", s.c_str());
	if (size == -1) {
		connected = false;
		throw cTcpException(cTcpException::ERR_CONNECTION_RESET);
	}
}

cTcpClient &cTcpClient::operator<<(std::string s) {
	Write(s);
	return *this;
}

cTcpClient &cTcpClient::operator<<(char c[]) {
	Write(std::string(c));
	return *this;
}

cTcpClient &cTcpClient::operator<<(int i) {
	std::ostringstream sData;
	sData << i;
	Write(sData.str());
	return *this;
}
