#ifndef TCPCLIENT_H_
#define TCPCLIENT_H_

#include <vdr/thread.h>
#include <string>
#include <sstream>
#include <stdexcept>

#define TCP_BUF_SIZE 4096

class cTcpException : public std::exception {
public:
	enum errorCode {
		ERR_HOST_NOT_RESOLVABLE,
		ERR_HOST_NOT_REACHABLE,
		ERR_SOCKET_CREATE,
		ERR_CONNECTION_RESET
	} errcode;

	cTcpException(errorCode errcode) {
		this->errcode = errcode;
	}

	virtual const char* what() const throw() {
		switch (errcode) {
		case ERR_HOST_NOT_RESOLVABLE:
			return "Host is not resolvable.";
		case ERR_HOST_NOT_REACHABLE:
			return "Host is not reachable.";
		case ERR_SOCKET_CREATE:
			return "Could not create socket.";
		case ERR_CONNECTION_RESET:
			return "Connection was reset.";
		default:
			return "Unknown cTcpException happened.";
		}

	}
};

class cTcpClient {
private:
	char buffer[TCP_BUF_SIZE];
	bool connected;
	int fd;
	std::string hostname;
	int port;
	std::string Read();
	void Write(std::string s);
	void Connect();
public:
	cTcpClient(std::string &hostname, int port);
	virtual ~cTcpClient();
	// ">>" is a blocking operation 
	cTcpClient &operator>>(std::ostringstream &ss);
	cTcpClient &operator>>(std::string &s);
	cTcpClient &operator<<(std::string s);
	cTcpClient &operator<<(char c[]);
	cTcpClient &operator<<(int i);
};

#endif /*TCPCLIENT_H_*/
